==> cloudpos-common-classes-1.7.32.jar: com/floreantpos/model/Discount.java <==
/**
 * ************************************************************************
 * * The contents of this file are subject to the MRPL 1.2
 * * (the  "License"),  being   the  Mozilla   Public  License
 * * Version 1.1  with a permitted attribution clause; you may not  use this
 * * file except in compliance with the License. You  may  obtain  a copy of
 * * the License at http://www.floreantpos.org/license.html
 * * Software distributed under the License  is  distributed  on  an "AS IS"
 * * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * * License for the specific  language  governing  rights  and  limitations
 * * under the License.
 * * The Original Code is FLOREANT POS.
 * * The Initial Developer of the Original Code is OROCUBE LLC
 * * All portions are Copyright (C) 2015 OROCUBE LLC
 * * All Rights Reserved.
 * ************************************************************************
 */
package com.floreantpos.model;

import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.apache.commons.lang.StringUtils;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.floreantpos.model.base.BaseDiscount;
import com.floreantpos.model.ext.DiscountCondition;
import com.google.gson.Gson;
import com.google.gson.JsonObject;

@JsonIgnoreProperties(ignoreUnknown = true, value = { "originalValue", "discountCondition", "displayText" })
@XmlAccessorType(XmlAccessType.FIELD)
@XmlRootElement
public class Discount extends BaseDiscount implements TimedModel, PropertyContainer {
	private static final long serialVersionUID = 1L;

	private static final String LOYALTY_DISCOUNT_ID = "loyalty_discount"; //$NON-NLS-1$
	public static final String LOYALTY_DISCOUNT_NAME = "Loyalty"; //$NON-NLS-1$
	public static final String LOYALTY_DISCOUNT_BARCODE = "loyalty"; //$NON-NLS-1$
	public static final String FOREVER = "Forever";//$NON-NLS-1$
	public static final String ONCE = "Once";//$NON-NLS-1$
	public static final String MULTIPLE_MONTHS = "Multiple months";//$NON-NLS-1$

	/*[CONSTRUCTOR MARKER BEGIN]*/
	public Discount() {
	}

	/**
	 * Constructor for primary key
	 */
	public Discount(java.lang.String id) {
		super(id);
	}

	/*[CONSTRUCTOR MARKER END]*/

	public final static int FREE_AMOUNT = 0;
	public final static int FIXED_PER_CATEGORY = 1;
	public final static int FIXED_PER_ITEM = 2;
	public final static int FIXED_PER_ORDER = 3;
	public final static int PERCENTAGE_PER_CATEGORY = 4;
	public final static int PERCENTAGE_PER_ITEM = 5;
	public final static int PERCENTAGE_PER_ORDER = 6;

	public final static int DISCOUNT_TYPE_AMOUNT = 0;
	public final static int DISCOUNT_TYPE_PERCENTAGE = 1;
	public final static int DISCOUNT_TYPE_REPRICE = 2;
	public final static int DISCOUNT_TYPE_LOYALTY = 3;
	public final static int DISCOUNT_TYPE_ALT_PRICE = 4;

	public final static int QUALIFICATION_TYPE_ITEM = 0;
	public final static int QUALIFICATION_TYPE_ORDER = 1;

	public final static String[] COUPON_TYPE_NAMES = { "AMOUNT", "PERCENTAGE", "REPRICE", "LOYALTY" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

	public final static String[] COUPON_QUALIFICATION_NAMES = { "ITEM", "ORDER" }; //$NON-NLS-1$ //$NON-NLS-2$
	private List<String> menuItemIds;
	private boolean updateLastUpdateTime = true;
	private boolean updateSyncTime = false;
	private double originalValue;
	private boolean expire;
	private transient JsonObject propertiesContainer;

	public boolean isUpdateSyncTime() {
		return updateSyncTime;
	}

	public void setUpdateSyncTime(boolean shouldUpdateSyncTime) {
		this.updateSyncTime = shouldUpdateSyncTime;
	}

	public boolean isUpdateLastUpdateTime() {
		return updateLastUpdateTime;
	}

	public void setUpdateLastUpdateTime(boolean shouldUpdateUpdateTime) {
		this.updateLastUpdateTime = shouldUpdateUpdateTime;
	}

	@Override
	public String toString() {
		return super.getName();
	}

	public double calculateDiscount(ITicketItem ticketItem) {

		Double unitPrice = ticketItem.getUnitPrice();
		switch (getType()) {
			case DISCOUNT_TYPE_AMOUNT:
				return unitPrice;

			case DISCOUNT_TYPE_PERCENTAGE:
				if (unitPrice != null) {
					return (getValue() * unitPrice) / 100;
				}
				return 0;

			default:
				break;
		}

		return 0;
	}

	public double getAmountByType(double price) {
		switch (getType()) {
			case Discount.DISCOUNT_TYPE_AMOUNT:
				return price - getValue();

			case Discount.DISCOUNT_TYPE_PERCENTAGE:
				return price - (getValue() / 100);

			default:
				break;
		}
		return 0;
	}

	public List<String> getMenuItemIds() {
		return menuItemIds;
	}

	public void setMenuItemIds(List<String> menuItemIds) {
		this.menuItemIds = menuItemIds;
	}

	public static String getLoyaltyDiscountId() {
		return LOYALTY_DISCOUNT_ID;
	}

	public String getTypeName() {
		return Discount.COUPON_TYPE_NAMES[getType()];
	}

	public void setTypeName(String typeName) {
	}

	public String getQualificationName() {
		return Discount.COUPON_QUALIFICATION_NAMES[getQualificationType()];
	}

	public void setQualificationName(String qualificationName) {
	}

	@Override
	public String getProperties() {
		if (propertiesContainer != null) {
			return propertiesContainer.toString();
		}

		String properties = super.getProperties();
		if (StringUtils.isEmpty(properties)) {
			return null;
		}

		propertiesContainer = new Gson().fromJson(properties, JsonObject.class);
		return properties;
	}

	@Override
	public void setProperties(String properties) {
		super.setProperties(properties);
		propertiesContainer = new Gson().fromJson(properties, JsonObject.class);
	}

	@Override
	public JsonObject getPropertyStore() {
		if (propertiesContainer == null) {
			propertiesContainer = new JsonObject();
		}
		return propertiesContainer;
	}

	@XmlTransient
	public double getOriginalValue() {
		return originalValue;
	}

	public void setOriginalValue(double originalValue) {
		this.originalValue = originalValue;
	}

	public void setDiscountCondition(DiscountCondition discountCondition) {
		if (discountCondition != null) {
			addProperty("discount.condition", discountCondition.name()); //$NON-NLS-1$
		}
		else {
			removeProperty("discount.condition"); //$NON-NLS-1$
		}
	}

	@XmlTransient
	public DiscountCondition getDiscountCondition() {
		String discountCondition = getProperty("discount.condition"); //$NON-NLS-1$
		if (StringUtils.isBlank(discountCondition)) {
			return null;
		}
		return DiscountCondition.fromValue(discountCondition);
	}

	@XmlTransient
	public String getDisplayText() {
		return getProperty("display_text"); //$NON-NLS-1$
	}

	public void setDisplayText(String displayText) {
		addProperty("display_text", displayText); //$NON-NLS-1$
	}

	@XmlTransient
	@JsonIgnore
	public Boolean isDefault() {
		return getBooleanProperty("default", false); //$NON-NLS-1$
	}

	public void setDefault(Boolean defaultCoupon) {
		addProperty("default", String.valueOf(defaultCoupon)); //$NON-NLS-1$
	}

	@XmlTransient
	@JsonIgnore
	public String getDurationType() {
		return getProperty("durationType"); //$NON-NLS-1$
	}

	public void setDurationType(String durationType) {
		addProperty("durationType", durationType); //$NON-NLS-1$
	}

	@XmlTransient
	@JsonIgnore
	public int getDurationInMonth() {
		return getIntProperty("durationInMonth"); //$NON-NLS-1$
	}

	public void setDurationInMonth(int durationType) {
		addProperty("durationInMonth", String.valueOf(durationType)); //$NON-NLS-1$
	}

	public boolean isExpire() {
		return expire;
	}

	public void setExpire(boolean expire) {
		this.expire = expire;
	}

}

==> cloudpos-common-classes-1.7.37-20230822.124515-10.jar: com/floreantpos/model/Discount.java <==
/**
 * ************************************************************************
 * * The contents of this file are subject to the MRPL 1.2
 * * (the  "License"),  being   the  Mozilla   Public  License
 * * Version 1.1  with a permitted attribution clause; you may not  use this
 * * file except in compliance with the License. You  may  obtain  a copy of
 * * the License at http://www.floreantpos.org/license.html
 * * Software distributed under the License  is  distributed  on  an "AS IS"
 * * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * * License for the specific  language  governing  rights  and  limitations
 * * under the License.
 * * The Original Code is FLOREANT POS.
 * * The Initial Developer of the Original Code is OROCUBE LLC
 * * All portions are Copyright (C) 2015 OROCUBE LLC
 * * All Rights Reserved.
 * ************************************************************************
 */
package com.floreantpos.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.apache.commons.lang.StringUtils;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.floreantpos.model.base.BaseDiscount;
import com.floreantpos.model.ext.DiscountCondition;
import com.floreantpos.util.NumberUtil;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

@JsonIgnoreProperties(ignoreUnknown = true, value = { "originalValue", "discountCondition", "displayText" })
@XmlAccessorType(XmlAccessType.FIELD)
@XmlRootElement
public class Discount extends BaseDiscount implements TimedModel, PropertyContainer {
	private static final long serialVersionUID = 1L;

	private static final String LOYALTY_DISCOUNT_ID = "loyalty_discount"; //$NON-NLS-1$
	public static final String LOYALTY_DISCOUNT_NAME = "Loyalty"; //$NON-NLS-1$
	public static final String LOYALTY_DISCOUNT_BARCODE = "loyalty"; //$NON-NLS-1$
	public static final String FOREVER = "Forever";//$NON-NLS-1$
	public static final String ONCE = "Once";//$NON-NLS-1$
	public static final String MULTIPLE_MONTHS = "Multiple months";//$NON-NLS-1$

	/*[CONSTRUCTOR MARKER BEGIN]*/
	public Discount() {
	}

	/**
	 * Constructor for primary key
	 */
	public Discount(java.lang.String id) {
		super(id);
	}

	/*[CONSTRUCTOR MARKER END]*/

	public final static int FREE_AMOUNT = 0;
	public final static int FIXED_PER_CATEGORY = 1;
	public final static int FIXED_PER_ITEM = 2;
	public final static int FIXED_PER_ORDER = 3;
	public final static int PERCENTAGE_PER_CATEGORY = 4;
	public final static int PERCENTAGE_PER_ITEM = 5;
	public final static int PERCENTAGE_PER_ORDER = 6;

	public final static int DISCOUNT_TYPE_AMOUNT = 0;
	public final static int DISCOUNT_TYPE_PERCENTAGE = 1;
	public final static int DISCOUNT_TYPE_REPRICE = 2;
	public final static int DISCOUNT_TYPE_LOYALTY = 3;
	public final static int DISCOUNT_TYPE_ALT_PRICE = 4;

	public final static int QUALIFICATION_TYPE_ITEM = 0;
	public final static int QUALIFICATION_TYPE_ORDER = 1;

	public final static String[] COUPON_TYPE_NAMES = { "AMOUNT", "PERCENTAGE", "REPRICE", "LOYALTY" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

	public final static String[] COUPON_QUALIFICATION_NAMES = { "ITEM", "ORDER" }; //$NON-NLS-1$ //$NON-NLS-2$
	private List<String> menuItemIds;
	private boolean updateLastUpdateTime = true;
	private boolean updateSyncTime = false;
	private double originalValue;
	private boolean expire;
	private transient JsonObject propertiesContainer;

	public boolean isUpdateSyncTime() {
		return updateSyncTime;
	}

	public void setUpdateSyncTime(boolean shouldUpdateSyncTime) {
		this.updateSyncTime = shouldUpdateSyncTime;
	}

	public boolean isUpdateLastUpdateTime() {
		return updateLastUpdateTime;
	}

	public void setUpdateLastUpdateTime(boolean shouldUpdateUpdateTime) {
		this.updateLastUpdateTime = shouldUpdateUpdateTime;
	}

	@Override
	public String toString() {
		return super.getName();
	}

	public double calculateDiscount(ITicketItem ticketItem) {

		Double unitPrice = ticketItem.getUnitPrice();
		switch (getType()) {
			case DISCOUNT_TYPE_AMOUNT:
				return unitPrice;

			case DISCOUNT_TYPE_PERCENTAGE:
				if (unitPrice != null) {
					return (getValue() * unitPrice) / 100;
				}
				return 0;

			default:
				break;
		}

		return 0;
	}

	public double getAmountByType(double price) {
		switch (getType()) {
			case Discount.DISCOUNT_TYPE_AMOUNT:
				return price - getValue();

			case Discount.DISCOUNT_TYPE_PERCENTAGE:
				return price - (getValue() / 100);

			default:
				break;
		}
		return 0;
	}

	public List<String> getMenuItemIds() {
		return menuItemIds;
	}

	public void setMenuItemIds(List<String> menuItemIds) {
		this.menuItemIds = menuItemIds;
	}

	public static String getLoyaltyDiscountId() {
		return LOYALTY_DISCOUNT_ID;
	}

	public String getTypeName() {
		return Discount.COUPON_TYPE_NAMES[getType()];
	}

	public void setTypeName(String typeName) {
	}

	public String getQualificationName() {
		return Discount.COUPON_QUALIFICATION_NAMES[getQualificationType()];
	}

	public void setQualificationName(String qualificationName) {
	}

	@Override
	public String getProperties() {
		if (propertiesContainer != null) {
			return propertiesContainer.toString();
		}

		String properties = super.getProperties();
		if (StringUtils.isEmpty(properties)) {
			return null;
		}

		propertiesContainer = new Gson().fromJson(properties, JsonObject.class);
		return properties;
	}

	@Override
	public void setProperties(String properties) {
		super.setProperties(properties);
		propertiesContainer = new Gson().fromJson(properties, JsonObject.class);
	}

	@Override
	public JsonObject getPropertyStore() {
		if (propertiesContainer == null) {
			propertiesContainer = new JsonObject();
		}
		return propertiesContainer;
	}

	@XmlTransient
	public double getOriginalValue() {
		return originalValue;
	}

	public void setOriginalValue(double originalValue) {
		this.originalValue = originalValue;
	}

	public void setDiscountCondition(DiscountCondition discountCondition) {
		if (discountCondition != null) {
			addProperty("discount.condition", discountCondition.name()); //$NON-NLS-1$
		}
		else {
			removeProperty("discount.condition"); //$NON-NLS-1$
		}
	}

	@XmlTransient
	public DiscountCondition getDiscountCondition() {
		String discountCondition = getProperty("discount.condition"); //$NON-NLS-1$
		if (StringUtils.isBlank(discountCondition)) {
			return null;
		}
		return DiscountCondition.fromValue(discountCondition);
	}

	@XmlTransient
	public String getDisplayText() {
		return getProperty("display_text"); //$NON-NLS-1$
	}

	public void setDisplayText(String displayText) {
		addProperty("display_text", displayText); //$NON-NLS-1$
	}

	@XmlTransient
	@JsonIgnore
	public Boolean isDefault() {
		return getBooleanProperty("default", false); //$NON-NLS-1$
	}

	public void setDefault(Boolean defaultCoupon) {
		addProperty("default", String.valueOf(defaultCoupon)); //$NON-NLS-1$
	}

	@XmlTransient
	@JsonIgnore
	public String getDurationType() {
		return getProperty("durationType"); //$NON-NLS-1$
	}

	public void setDurationType(String durationType) {
		addProperty("durationType", durationType); //$NON-NLS-1$
	}

	@XmlTransient
	@JsonIgnore
	public int getDurationInMonth() {
		return getIntProperty("durationInMonth"); //$NON-NLS-1$
	}

	public void setDurationInMonth(int durationType) {
		addProperty("durationInMonth", String.valueOf(durationType)); //$NON-NLS-1$
	}

	public boolean isExpire() {
		return expire;
	}

	public void setExpire(boolean expire) {
		this.expire = expire;
	}

	public String getDiscountValueDisplay() {
		if (getType().intValue() == DISCOUNT_TYPE_PERCENTAGE) {
			return NumberUtil.trimDecilamIfNotNeeded(super.getValue()) + "%"; //$NON-NLS-1$
		}
		return NumberUtil.trimDecilamIfNotNeeded(super.getValue());
	}

	public void setDiscountValueDisplay(String discountValueDisplay) {
	}

	public List<String> getCustomerIds() {
		List<String> ids = new ArrayList<>();
		JsonArray jsonArray = getPropertyStore().getAsJsonArray("customerIds"); //$NON-NLS-1$
		if (jsonArray != null && jsonArray.size() > 0) {
			for (int i = 0; i < jsonArray.size(); i++) {
				JsonElement jsonElmt = jsonArray.get(i);
				String id = jsonElmt.getAsString();
				if (StringUtils.isBlank(id)) {
					continue;
				}
				ids.add(id);
			}
		}
		return ids;
	}

	public List<String> getResellerIds() {
		List<String> ids = new ArrayList<>();
		JsonArray jsonArray = getPropertyStore().getAsJsonArray("resellerIds"); //$NON-NLS-1$
		if (jsonArray != null && jsonArray.size() > 0) {
			for (int i = 0; i < jsonArray.size(); i++) {
				JsonElement jsonElmt = jsonArray.get(i);
				String id = jsonElmt.getAsString();
				if (StringUtils.isBlank(id)) {
					continue;
				}
				ids.add(id);
			}
		}
		return ids;
	}

	public void setCustomers(Collection customers) {
		JsonArray jsonArray = new JsonArray();
		if (customers != null) {
			for (Iterator iterator = customers.iterator(); iterator.hasNext();) {
				Customer c = (Customer) iterator.next();
				jsonArray.add(c.getId());
			}
		}
		getPropertyStore().add("customerIds", jsonArray); //$NON-NLS-1$
	}

	public void setResellers(Collection resellers) {
		JsonArray jsonArray = new JsonArray();
		if (resellers != null) {
			for (Iterator iterator = resellers.iterator(); iterator.hasNext();) {
				Customer c = (Customer) iterator.next();
				jsonArray.add(c.getId());
			}
		}
		getPropertyStore().add("resellerIds", jsonArray); //$NON-NLS-1$
	}
}
